# AUTOGENERATED! DO NOT EDIT! File to edit: ../../nbs/PackageData.ipynb.

# %% auto 0
__all__ = ['get_root_data_path']

# %% ../../nbs/PackageData.ipynb 1
from pathlib import Path
from typing import *

# %% ../../nbs/PackageData.ipynb 2
def get_root_data_path() -> Path:
    """Returns path to package_data in lib

    Example:
        ``` python
        from nbdev_mkdocs._package_data import get_root_data_path

        mkdocs_template_path = get_root_data_path() / "mkdocs_template.yml"
        print(f"Path is: {mkdocs_template_path.resolve()}")
        assert mkdocs_template_path.exists()

        ```

    !!! note

        The above docstring is autogenerated by docstring-gen library (https://github.com/airtai/docstring-gen)
    """
    return Path(__file__).parent.parent / "package_data"
